import * as React from 'react'
import { Provider } from 'react-redux'
import { HashRouter as Router, Route, Switch  } from 'react-router-dom'
import { PersistGate } from 'redux-persist/integration/react'
import './App.less'
import { appContainer } from './redux/connect'
import { store } from './redux/store'
import { persistor } from './redux/store'
import { router } from './router'
import { loadable } from './router'

interface IRouterItem {
  path: string
  component: Promise<React.ComponentClass<any> | React.StatelessComponent<any> | { default: React.ComponentType<any> }>
}

class App extends React.Component<any, any> {
  constructor (props: any) {
    super(props)
    this.getRouter = this.getRouter.bind(this)
  }
  public getRouter() {
    const routerArr = router.map((item: IRouterItem, index: number) => {
      return (
        <Route path={item.path} component={loadable(item.component)} key={index}/>
      )
    })
    return routerArr
  }
  public render() {
    return (
      <div className="App">
        <Provider store={store}>
          <PersistGate loading={null} persistor={persistor}>
            <Router>
              <Switch>
                <Route path='/' exact={true} component={appContainer} />
                {
                  this.getRouter()
                }
              </Switch>
            </Router>
          </PersistGate>
        </Provider>
        
      </div>
    )
  }
}

export default App
